#include "CPreferences.h"

#include <FindDirectory.h>
#include <Directory.h>
#include <Path.h>
#include <Entry.h>
#include <File.h>
#include <Control.h>

CPreferences::CPreferences()
{
	BPath		path;
	BEntry		settingPathEntry;
	
	// par defaut on a pas reussi
	_status = B_ERROR;

	// trouver si le repertoire Settings existe
	if(find_directory(B_USER_SETTINGS_DIRECTORY,&path)!=B_OK)
		return ;

	// verifier si le repertoire DynDnsUpdate existe
	// sinon le creer
	_filePath = path.Path();
	_filePath << "/DynDnsUpdate";
	settingPathEntry.SetTo(_filePath.String());
	if(!(settingPathEntry.Exists()))
		create_directory(_filePath.String(),00777);

	_filePath << "/DynDnsUpdate.prefs";
	_status = B_OK;
}

/**** destructeur ****/
CPreferences::~CPreferences()
{
}

/**** charger les preferences ****/
status_t CPreferences::Load()
{
	status_t	state = B_OK;
	BFile		file;

	if((_status=file.SetTo(_filePath.String(),B_READ_ONLY))!=B_OK)
		return _status;
	
	// charger les donnees dans le message
	BMessage::MakeEmpty();
	state &= BMessage::Unflatten(&file);
	state &= BMessage::FindString("ip-cache",&_Ip);
	state &= BMessage::FindString("user",&_user);
	state &= BMessage::FindString("password",&_password);
	state &= BMessage::FindString("host",&_host);
	state &= BMessage::FindString("system",&_system);
	state &= BMessage::FindInt32("wildcard",&_wildcard);
	state &= BMessage::FindString("mx",&_mx);
	state &= BMessage::FindInt32("backmx",&_backmx);
	state &= BMessage::FindInt32("offline",&_offline);
	state &= BMessage::FindString("application-path",&_applicationPath);
	state &= BMessage::FindString("check-ip-server",&_checkIpServer);
	state &= BMessage::FindString("dyndns-server",&_dyndnsServer);
	state &= BMessage::FindInt32("port",&_port);
	state &= BMessage::FindInt32("check-delay",&_delay);
	state &= BMessage::FindInt8("time-out",&_timeOut);
	state &= BMessage::FindInt32("log-size",&_logSize);
	state &= BMessage::FindRect("settings-win-position",&_winPosition);
		
	return state;
}

/**** sauver les preferences ****/
status_t CPreferences::Save()
{
	BFile		file;
	status_t	status;

	if((status=file.SetTo(_filePath.String(),B_CREATE_FILE|B_ERASE_FILE|B_WRITE_ONLY))!=B_OK)
		return status;
	
	// Sauver les donnees du message	
	BMessage::MakeEmpty();
	BMessage::AddString("ip-cache",_Ip);
	BMessage::AddString("user",_user);
	BMessage::AddString("password",_password);
	BMessage::AddString("host",_host);
	BMessage::AddString("system",_system);
	BMessage::AddInt32("wildcard",_wildcard);
	BMessage::AddString("mx",_mx);
	BMessage::AddInt32("backmx",_backmx);
	BMessage::AddInt32("offline",_offline);
	BMessage::AddString("application-path",_applicationPath);
	BMessage::AddString("check-ip-server",_checkIpServer);
	BMessage::AddString("dyndns-server",_dyndnsServer);
	BMessage::AddInt32("port",_port);
	BMessage::AddInt32("check-delay",_delay);
	BMessage::AddInt8("time-out",_timeOut);
	BMessage::AddInt32("log-size",_logSize);
	BMessage::AddRect("settings-win-position",_winPosition);
	status = BMessage::Flatten(&file);
	
	return status;
}

/**** default properties ****/
status_t CPreferences::SetDefault()
{
	// default !!!! for test
	_Ip = "0.0.0.0";
	_user = "no user";
	_password = "*****";
	_host = "empty";
	_system = "dyndns";
	_wildcard = B_CONTROL_OFF;
	_mx = "";
	_backmx = B_CONTROL_OFF;
	_offline = B_CONTROL_OFF;
	_checkIpServer = "checkip.dyndns.org";
	_dyndnsServer = "members.dyndns.org";
	_timeOut = 10;
	_delay = 300;
	_port = 80;
	_logSize = 4096;
	_winPosition.Set(50,50,311,240);
	
	// save default
	Save();

	return B_OK;
}

/**** verifier si les prefs sont chargé ****/
status_t CPreferences::PreferencesLoaded()
{
	return _status;
}

/**** recuperer le chemin ****/
const char	*CPreferences::SettingPath()
{
	return	_filePath.String();
}