#include "DynDnsUpdateReplicant.h"
#include "DynDnsSettingsWin.h"
#include "DynDnsUpdate.h"
#include "CPreferences.h"

#include <Message.h>
#include <Deskbar.h>
#include <Bitmap.h>
#include <Resources.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Roster.h>

#include <stdio.h>

// extern function !
extern "C" _EXPORT BView* instantiate_deskbar_item();

BView* instantiate_deskbar_item(void)
{
	return new DynDnsUpdateReplicant();
}

/**** Constructor ****/
DynDnsUpdateReplicant::DynDnsUpdateReplicant()
: BView(BRect(0,0,15,15),"DynDnsUpdateReplicant", B_FOLLOW_NONE, B_WILL_DRAW | B_PULSE_NEEDED)
{
}

/**** Constructor with BMessage ****/
DynDnsUpdateReplicant::DynDnsUpdateReplicant(BMessage *message)
: BView(message)
{
}

/**** Destructor ****/
DynDnsUpdateReplicant::~DynDnsUpdateReplicant()
{
}

/**** instanciate new view ****/
DynDnsUpdateReplicant *DynDnsUpdateReplicant::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data,"DynDnsUpdateReplicant"))
		return NULL;
	
	return (new DynDnsUpdateReplicant(data));
}

/**** archiving view ****/
status_t DynDnsUpdateReplicant::Archive(BMessage *data, bool deep = true) const
{
	BView::Archive(data, deep);
	data->AddString("add_on","application/x-vnd.CKJ.DynDnsUpdate");

	return B_NO_ERROR;
}

/**** Attached to window ****/
void DynDnsUpdateReplicant::AttachedToWindow()
{
	CPreferences	prefs;

	// loading prefs
	prefs.Load();
		
	// loading all Bitmaps
	_normalBitmap = GetBitmap("DDU:Normal",'BBMP');
	_pendingBitmap[0] = _normalBitmap;
	_pendingBitmap[1] = GetBitmap("DDU:Pending1",'BBMP');
	_pendingBitmap[2] = GetBitmap("DDU:Pending2",'BBMP');
	_pendingBitmap[3] = GetBitmap("DDU:Pending3",'BBMP');
	_errorBitmap = GetBitmap("DDU:Error",'BBMP');

	// initialize
	_delay = prefs._delay;
	_lastPulse = time(0);
	_cmpt = 0;
	_displayedBitmap = _normalBitmap;

	// call parent function
	BView::AttachedToWindow();

	// start DynDns Looper thread
	_dyndns = new DynDnsUpdate();
	_dyndns->Run();
	
	// build popupmenu
	BuildMenu();
	
	// setting window
	_settingWin = new DynDnsSettingsWin(prefs._winPosition,_dyndns);
}	


/**** Detached from window ****/
void DynDnsUpdateReplicant::DetachedFromWindow()
{
	CPreferences	prefs;

	// loading prefs
	prefs.Load();

	// call parent function
	BView::DetachedFromWindow();

	if(_normalBitmap!=NULL)
		delete _normalBitmap;
	
	if(_errorBitmap!=NULL)
		delete _errorBitmap;
	
	// dont delete the first, it's the normal bitmap
	for(int8 i=1;i<MAX_BBMP_PENDING;i++)
		if(_pendingBitmap[i]!=NULL)
			delete _pendingBitmap[i];
	
	// free memory take by popup
	delete _optionMenu;
	
	// quit and destroy the looper
	_dyndns->Lock();
	_dyndns->Quit();
	
	// save window position
	prefs._winPosition = _settingWin->Frame();
	prefs.Save();

	// quit setting window
	_settingWin->PostMessage(B_QUIT_REQUESTED);
}

/**** draw the view ****/
void DynDnsUpdateReplicant::Draw(BRect updateRect)
{
	// color of the deskbar tray view
	SetHighColor(tint_color(ui_color(B_MENU_BACKGROUND_COLOR),B_DARKEN_1_TINT));
	FillRect(Bounds());
	
	// display bitmap
	if(_normalBitmap!=NULL)
	{
		SetDrawingMode(B_OP_OVER);
		DrawBitmap(_displayedBitmap,BPoint(0,0));
		SetDrawingMode(B_OP_COPY);
	}
}

/**** Mouse click ****/
void DynDnsUpdateReplicant::MouseDown(BPoint point)
{
	// we must no have network operation
	if(_dyndns->IsPending())
		return;
		
	// display popupmenu
	BMenuItem *selected = _optionMenu->Go(ConvertToScreen(point));
	switch(_optionMenu->IndexOf(selected))
	{
	case 0:
		if(_settingWin->IsHidden())
			_settingWin->Show();
		break;
	case 1:
		// force Ip Check, just modify last update time
		_lastPulse = time(0) - _delay;
		break;
	case 2:
		// force update, just post update message
		_dyndns->PostMessage(DYNDNS_IPUPDATE_MSG);
		break;
	case 4:
		{
			// remove replicant
			BDeskbar	desk;
			desk.RemoveItem("DynDnsUpdateReplicant");
		}
		break;
	}
}

/**** Pulse ****/
void DynDnsUpdateReplicant::Pulse()
{
	time_t pulseTime = time(0);
	
	// verify if you must execute action
	// only if the delay you set is passed
	if(pulseTime >= (_lastPulse+_delay))
	{
		// we must have hide setting window before connect to server
		// that save settings (include in Hide windows function)
		// checkIp and if that change, update-it
		if(_settingWin->IsHidden())
			_dyndns->PostMessage(DYNDNS_IPCHECK_MSG);
	
		// update last Pulse time
		_lastPulse = pulseTime;
	}

	// network communication
	// we must manage the pending animation here
	if(_dyndns->IsPending())
	{
		if(_cmpt==MAX_BBMP_PENDING)
			_cmpt = 0;
			
		_displayedBitmap = 	_pendingBitmap[_cmpt];
		Invalidate();
		
		_cmpt++;
	}
	else
	{
		// display normal bitmap or error bitmap
		if(_displayedBitmap!=_normalBitmap && _displayedBitmap!=_errorBitmap)
		{
			_displayedBitmap = _normalBitmap;
			if(_dyndns->CheckIpStatus()!=B_OK || _dyndns->UpdateIpStatus()!=B_OK)
				_displayedBitmap = _errorBitmap;

			// refresh screen
			Invalidate();
			_cmpt = 0;
		}
	}
}
/**** find bitmap into ressource ****/
BBitmap *DynDnsUpdateReplicant::GetBitmap(const char *name,int32 type)
{
	CPreferences	prefs;
	BFile			file;
	BBitmap 		*bitmap = NULL;
	size_t 			len = 0;
	status_t 		error;	
	BResources		resources;

	// load prefs for application path
	if(prefs.Load()!=B_OK)
		return NULL;

	// set ressource and load
	file.SetTo(prefs._applicationPath.String(),B_READ_ONLY);
	resources.SetTo(&file);
	const void *data = resources.LoadResource(type, name, &len);

	BMemoryIO stream(data, len);
	
	// archived Bitmap
	BMessage archive;
	error = archive.Unflatten(&stream);
	if (error != B_OK)
		return NULL;

	// try to create BBitmap fro archive
	bitmap = new BBitmap(&archive);
	if(!bitmap)
		return NULL;

	// ok ?
	if(bitmap->InitCheck() != B_OK)
	{
		delete bitmap;
		return NULL;
	}
	
	// all is good
	return bitmap;
}

/**** build popupmenu ****/
void DynDnsUpdateReplicant::BuildMenu()
{
	_optionMenu = new BPopUpMenu("Options");
	_optionMenu->AddItem(new BMenuItem("Settings",NULL));
	_optionMenu->AddItem(new BMenuItem("Force Ip Check",NULL));
	_optionMenu->AddItem(new BMenuItem("Force Update",NULL));
	_optionMenu->AddSeparatorItem();
	_optionMenu->AddItem(new BMenuItem("Exit",NULL));	
}
