/**********************/
/* Window od settings */
/**********************/
#include "DynDnsSettingsWin.h"
#include "DynDnsUpdate.h"
#include "MTitleView.h"
#include "MTabView.h"

// setting views
#include "AboutView.h"
#include "UserView.h"
#include "ServerView.h"
#include "LogView.h"
#include "OtherView.h"

#include <TabView.h>
#include <TextView.h>

#include <stdio.h>

/**** constructor ****/
DynDnsSettingsWin::DynDnsSettingsWin(BRect frame,DynDnsUpdate *updater)
: BWindow(frame,"Settings",B_NO_BORDER_WINDOW_LOOK,B_NORMAL_WINDOW_FEEL, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
{
	BRect	bounds;

	// get DynDnsUpdater
	_updater = updater;

	// title view
	bounds = Bounds();
	bounds.bottom = 7;
	_titleView = new MTitleView(bounds);
	AddChild(_titleView);

	// Add tabView
	bounds = Bounds();
	bounds.top += 8;
	_tabView = new MTabView(bounds,"settings-tab");
	
	// setting View
	bounds = Bounds();
	bounds.bottom -= 31;
	bounds.right -= 2;
	_aboutView = new AboutView(bounds);
	_userView = new UserView(bounds);
	_serverView = new ServerView(bounds);
	_otherView = new OtherView(bounds);
	_logView = new LogView(bounds);
	
	_tabView->AddTab(_aboutView);
	_tabView->AddTab(_userView);
	_tabView->AddTab(_serverView);
	_tabView->AddTab(_otherView);
	_tabView->AddTab(_logView);

	AddChild(_tabView);
}

/**** destructor ****/
DynDnsSettingsWin::~DynDnsSettingsWin()
{
}	

/**** message ****/
void DynDnsSettingsWin::MessageReceived(BMessage *message)
{
	BWindow::MessageReceived(message);
}

/*** Show windows ****/
void DynDnsSettingsWin::Show()
{
	// update log view
	if(_logView!=NULL && _updater!=NULL)
	{
		Lock();
		_logView->_logTextView->Clear();
		_logView->_logTextView->Insert(_updater->Log());
		Unlock();
	}
	
	// default function
	BWindow::Show();
}

/*** Hide windows ****/
void DynDnsSettingsWin::Hide()
{
	// call when user ues ESC key
	// we must save settings
	_userView->SaveSettings();
	_serverView->SaveSettings();
	_otherView->SaveSettings();

	// default function
	BWindow::Hide();
}
