/**************/
/* Title View */
/**************/
#include "MTitleView.h"

#include <Window.h>
#include <Screen.h>

#include <stdio.h>

/**** constructor ****/
MTitleView::MTitleView(BRect frame)
: BView(frame,"title-view",B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	BScreen		screen;

	// screen size
	_screenSize = screen.Frame();

	// not move by default
	_moveWindow = false;

	// get all movement of cursor
	SetEventMask(B_KEYBOARD_EVENTS | B_POINTER_EVENTS,B_NO_POINTER_HISTORY);
}
	
/**** destructor ****/
MTitleView::~MTitleView()
{
}

/**** drawing the yellow rect ****/
void MTitleView::Draw(BRect updateRect)
{
	BRect	bounds = Bounds();

	SetHighColor(255,255,80);
	StrokeLine(BPoint(0,0),BPoint(bounds.right,0));
	StrokeLine(BPoint(0,1),BPoint(0,bounds.bottom));
	SetHighColor(255,203,0);
	FillRect(BRect(1,1,bounds.right-1,bounds.bottom));
	SetHighColor(175,123,0);
	StrokeLine(BPoint(bounds.right,1),BPoint(bounds.right,bounds.bottom));
}

/**** mouse click point ****/
void MTitleView::MouseDown(BPoint point)
{
	BRect	bounds = Bounds();
	
	// save click point
	_moveWindow = false;
	if(bounds.Contains(point))
	{
		_cliquePoint = ConvertToScreen(point);
		_frame = Window()->Frame();
		_moveWindow = true;
	}
}

/**** manage keyboard ****/
void MTitleView::KeyDown(const char *bytes, int32 numBytes)
{
	// hide window if you use ESC key
	if((int32)(*bytes)==27 && !(Window()->IsHidden()))
		Window()->Hide();
}

/**** fin de clique souris ****/
void MTitleView::MouseUp(BPoint point)
{
	// disable move option	
	_moveWindow = false;
}


/**** Mouse moved ****/
void MTitleView::MouseMoved(BPoint point,uint32 transit,const BMessage *message)
{
	uint32	button;
	BPoint	pt;

	// get mouse button state
	GetMouse(&pt,&button);

	// if we have click, move window
	if(button!=0 && _moveWindow && Window()->IsActive())
	{
		ConvertToScreen(&point);
		// verify we never go out from screen
		pt.x = point.x - _cliquePoint.x;
		pt.y = point.y - _cliquePoint.y;
		_frame.OffsetBy(pt.x,pt.y);
		// not go under 0
		if(_frame.left < _screenSize.left)
			_frame.OffsetTo(0,_frame.top);
		// not go over screen right
		if(_frame.right > _screenSize.right)
			_frame.OffsetTo((_screenSize.right-_frame.Width()),_frame.top);
		// not go under 0
		if(_frame.top < _screenSize.top)
			_frame.OffsetTo(_frame.left,0);
		// not go over screen right
		if(_frame.bottom > _screenSize.bottom)
			_frame.OffsetTo(_frame.left,(_screenSize.bottom-_frame.Height()));
		
		// move window
		Window()->MoveTo(_frame.left,_frame.top);
		_frame = Window()->Frame();
		_cliquePoint = point;
	}
}