#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "DynDnsUpdate.h"
#include "CPreferences.h"

#include <Control.h>

/**** constructeur ****/
DynDnsUpdate::DynDnsUpdate()
: BLooper("DynDnsUpdateLooper")
{
	// init
	_pending = false;
	_updateStatus = B_OK;
	_checkIpStatus = B_OK;
	
	// clear log
	_log.SetTo("");
}

/**** destructeur ****/
DynDnsUpdate::~DynDnsUpdate()
{
}

/**** Manage Messaging ****/
void DynDnsUpdate::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
	case DYNDNS_IPCHECK_MSG:
		// Check Ip
		_pending = true;
		_checkIpStatus = CheckIp();
		_pending = false;
		break;
	case DYNDNS_IPUPDATE_MSG:
		// Update with new Ip
		_pending = true;
		_updateStatus = Update();
		_pending = false;		
		break;
	default:
		BLooper::MessageReceived(message);
	}
}

/**** Update ****/
status_t DynDnsUpdate::Update()
{
	CPreferences	prefs;
	BString			request;
	BString			userPass;
	char			buffer;
	BString			answer;
	int32			offset;

	// check the log
	CheckLog();

	// loading settings
	if(prefs.Load()!=B_OK)
		return B_ERROR;

	if(prefs._Ip=="0.0.0.0")
	{
		Log("Error not good Ip in cache : 0.0.0.0\n");
		return B_ERROR;
	}

	// create string for user/password
	userPass = prefs._user;
	userPass << ":";
	userPass << prefs._password;

	// build string for update
	request = "GET /nic/update?system=";
	request << prefs._system;
	request << "&hostname=";
	request << prefs._host;
	request << "&myip=";
	request << prefs._Ip;
	
	// wildcard
	request << "&wildcard=";
	if(prefs._wildcard == B_CONTROL_ON)
		request << "ON";
	else
		request << "OFF";

	// mail exchanger
	request << "&mx=";
	request << prefs._mx;	
	// backmx
	request << "&backmx=";
	if(prefs._backmx == B_CONTROL_ON)
		request << "YES";
	else
		request << "NO";
	
	// offline
	request << "&offline=";
	if(prefs._offline == B_CONTROL_ON)
		request << "YES";
	else
		request << "NO";
	
	// end update string for dyndns
	request << " HTTP/1.1\r\n";
	// host
	request << "Host: ";
	request << prefs._dyndnsServer;
	request << "\r\n";
	// user + password
	request << "Authorization: Basic ";
	request << EncodeBase64(userPass.String());
	request << "\r\n";
	// client
	request << "User-Agent: ";
	request << CLIENT;
	request << " ";
	request << CLIENT_VERSION;
	request << " - ";
	request << CLIENT_DEVELOPPER;
	request << "\r\n";
	// option
	request << "Connection: close\r\n";
	request << "Pragma: no-cache\r\n";
	// fin
	request << "\r\n";

 	// init socket
 	_server.SetTo(prefs._dyndnsServer.String(),prefs._port);
	if(_member.InitCheck()!=B_OK)
	{
		Log("Error Server InitCheck()\n");
		return B_ERROR;
	}

	// time out in second
 	_member.SetTimeout(1000000 * prefs._timeOut);	
	if(_member.Connect(_server)==B_ERROR)
		Log("Error Server Connection\n");

	// send request for update
	printf("Request : %s\n",request.String());
	if(_member.Send((void *)(request.String()),request.Length(),0)!=request.Length())
	{
		// close socket
		_member.Close();
		
		Log("Error Send update-string");
		return B_ERROR;
	}
		
	// reception
	while(_member.Receive((void *)&buffer,1)==1)
		answer << buffer;

	// free memory and close socket
    _member.Close();

	// we must remove header (10 \r\n)
	offset = answer.FindFirst("Encoding",0);
	if(offset==B_ERROR)
	{
		// errrorin receive message
		Log("Data receiving Error !");
		
		return B_ERROR;
	}
	// remove header
	answer.Remove(0,offset+21);

	// remove all
	answer.RemoveSet("\n");
	answer.RemoveSet("\r");

	// remove the 0 (last char)
	answer.Remove(answer.Length()-1,1);
	
	// replace answer for more explain
	answer.ReplaceFirst("nochg","Not Changed (same ip)");
	answer.ReplaceFirst("good","Ip Changed");

	// log answer	
	Log(answer.String());
	
    
    // well all is ok
    return B_OK;
}

/**** recuperer l'Ip de votre paserelle ****/
status_t DynDnsUpdate::CheckIp()
{
	CPreferences	prefs;
	BString			request;
	BNetAddress		serverAddr;
	BNetEndpoint	server;
	char			buffer;
	BString			answer;
	int32			offset;
	int8			ipPart[4];

	// check the log
	CheckLog();

	// load old settings
	if(prefs.Load()!=B_OK)
		return B_ERROR;

	// build request
	request = "GET / HTTP/1.1\r\n";
	// host
	request << "Host: ";
	request << prefs._checkIpServer;
	request << "\r\n";
	// client
	request << "User-Agent: ";
	request << CLIENT;
	request << " ";
	request << CLIENT_VERSION;
	request << " - ";
	request << CLIENT_DEVELOPPER;
	request << "\r\n";
	// option
	request << "Connection: close\r\n";
	request << "Pragma: no-cache\r\n";
	// fin
	request << "\r\n";

	// BNetEndpoint must be initialized
	if(server.InitCheck()!=B_OK)
		return B_ERROR;

    // connect to checkip server
    printf("Resolving ip address...\n");
    serverAddr.SetTo(prefs._checkIpServer.String(),prefs._port);
 	server.SetTimeout(1000000 * prefs._timeOut);	
	if(server.Connect(serverAddr)!=B_OK)
	{
		Log("Error IpCheck-Server InitCheck()\n");
		return B_ERROR;
	}

	// send request for Ip
	if(server.Send((void *)(request.String()),request.Length())!=request.Length())
		Log("Error Send ip-check request");
	
	// reception
	while(server.Receive((void *)&buffer,1)==1)
		answer << buffer;

	// close socket and save option
    server.Close();
	
	// check to find ip result
	// it's possible don't find : timeout
	offset = answer.FindFirst("Current IP Address: ");
	if(offset==B_ERROR)
	{
		// save 0.0.0.0
		prefs._Ip = "0.0.0.0";
    	prefs.Save();
    	
		return B_ERROR;
	}

	offset += 20;
	answer.Remove(0,offset);
	offset = answer.FindFirst("<br>");
	answer.Remove(offset-1,answer.Length()-offset+1);
	if(prefs._Ip==answer)
	{
		Log("Ip not changed !\n");
		return B_OK;
	}

	// verify validity of new Ip
	sscanf(answer.String(),"%c.%c.%c.%c",&ipPart[0],&ipPart[1],&ipPart[2],&ipPart[3]);
	if(ipPart[0]==0 && ipPart[1] && ipPart[2] &&ipPart[3])
	{
		Log("Ip Corrupted : ");
		_log << answer.String();
		_log << "\n";
		return B_ERROR;
	}	
	
	// save new Ip
	prefs._Ip = answer;
    prefs.Save();

    // well all is ok
    Log("Your new IpAddress is : ");
    _log << prefs._Ip.String();
    _log << "\n";

	// all is ok
	// Update DynDns if IpChange
	PostMessage(DYNDNS_IPUPDATE_MSG);

    return B_OK;
}

/**** Encode Base 64 ****/
BString DynDnsUpdate::EncodeBase64(const char *buffer)
{
	char 		table64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	BString		result;
    
    unsigned char ibuf[3];
    unsigned char obuf[4];
    char output[4];
    int32 i;
    int32 inputparts;
   
    while(*buffer)
    {
	    for (i = inputparts = 0; i < 3; i++)
	    { 
			if(*buffer)
			{
				inputparts++;
			    ibuf[i] = *buffer;
			    buffer++;
		    }
	        else
		    	ibuf[i] = 0;
	    }
	
	    obuf [0] = (ibuf [0] & 0xFC) >> 2;
	    obuf [1] = ((ibuf [0] & 0x03) << 4) | ((ibuf [1] & 0xF0) >> 4);
	    obuf [2] = ((ibuf [1] & 0x0F) << 2) | ((ibuf [2] & 0xC0) >> 6);
	    obuf [3] = ibuf [2] & 0x3F;
	
	    switch(inputparts)
	    {
	    case 1: /* only one byte read */
	    	sprintf(output, "%c%c==",table64[obuf[0]],table64[obuf[1]]);
	        break;
	     case 2: /* two bytes read */
	        sprintf(output, "%c%c%c=",table64[obuf[0]],table64[obuf[1]],table64[obuf[2]]);
	        break;
	     default:
	        sprintf(output, "%c%c%c%c",table64[obuf[0]],table64[obuf[1]],table64[obuf[2]],table64[obuf[3]] );
	        break;
	    }
     	
     	// concat the result
     	result << output;
     }
     
     return result;
}

/**** Log error ****/
void DynDnsUpdate::Log(const char *trace)
{
	time_t	logTime;
	tm		logDate;
	
	logTime = time(0);
	logDate = *gmtime(&logTime);

	_log << logDate.tm_mday;
	_log << "/";
	_log << logDate.tm_mon;
	_log << "/";
	_log << logDate.tm_year;
	_log << "-";
	_log << logDate.tm_hour;
	_log << ":";
	_log << logDate.tm_min;
	_log << ":";
	_log << logDate.tm_sec;
	_log << " ";

	// add trace
	_log << trace;
}

/**** clear the log if we must ****/
void DynDnsUpdate::CheckLog()
{
	CPreferences	prefs;
	int32			size = DEFAULT_LOG_LENGTH;

	if(prefs.Load()==B_OK)
		size = prefs._logSize;
	
	if(_log.Length()>size)
		_log.SetTo("");
}